/*
 * Created on Nov 8, 2004
 *
 */
package jPDFViewerSamples;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.FlowLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import com.qoppa.pdf.FontSettings;
import com.qoppa.pdf.PDFException;
import com.qoppa.pdf.PrintSettings;
import com.qoppa.pdf.source.URLPDFSource;
import com.qoppa.pdfViewer.PDFViewerBean;
import com.qoppa.pdfViewer.panels.PDFPanel;
/**
 * @author Gerald Holmann
 *
 */
public class PDFViewer extends JFrame implements DropTargetListener
{
	private JPanel jPanel = null;
	private PDFViewerBean PDFViewer = null;
	private JPanel jpAPI = null;
	private JLabel jlJARLocation = null;
	private JButton jbViewAPI = null;
	private JLabel jLabel = null;

	private final static String OS_WINDOWS_START = "windows";  //  @jve:decl-index=0:
	private final static String OS_MAC = "mac";
	private final static String JAR_FILE_NAME = "jPDFViewer.jar";  //  @jve:decl-index=0:
	private final static String SAMPLES_DIR_NAME = "jPDFViewerSamples";  //  @jve:decl-index=0:
	private final static String API_INDEX_FILENAME = "javadoc/index.html";
	private final static String USER_GUIDE = "https://www.qoppa.com/files/pdfviewer/userguide/";

	private JMenuBar jMenuBar1 = null;
	private JMenu jmFile = null;
	private JMenu jmEdit = null;
	private JMenu jmView = null;
	private JMenu jmToolbar = null;
	private JMenuItem jmiOpen = null;
	private JMenuItem jmiOpenURL = null;
	private JMenuItem jmiPrint = null;
	private JSeparator jSeparator = null;
	private JMenuItem jmiClose = null;
	private JMenuItem jmiExit = null;
	private JCheckBoxMenuItem jcbShowToolbar = null;
	private JSeparator jSeparator1 = null;
	private JCheckBoxMenuItem jcbShowOpen = null;
	private JCheckBoxMenuItem jcbShowPrint = null;
	private JLabel jlSampleLocation = null;
    private JCheckBoxMenuItem jcbShowSelectToolbar = null;
    private JMenuItem jmiSearch = null;
    private JMenuItem jmiAdvSearch = null;
    private JMenu jmUserGuide = null;
    private JMenuItem jmiUserGuide = null;
    
    private JMenuItem jmiActualSize;
    private JMenuItem jmiFitToWidth;
    private JMenuItem jmiFitToPage;
    
    private JCheckBoxMenuItem jcbmiInvertColors;
    private boolean binvert = false;
    private JCheckBoxMenuItem jcbmiToggleTouch;
    private JMenuItem jmiPreviousView;
    private JMenuItem jmiNextView;
    private JMenu jmPageMode;
    private ButtonGroup bgPageMode;
    private JCheckBoxMenuItem jcmiPMContinuous;
    private JCheckBoxMenuItem jcmiPMSingle;
    private JCheckBoxMenuItem jcmiPMFacing;
    private JCheckBoxMenuItem jcmiPMFacingCont;
    private JCheckBoxMenuItem jcmiPMCover;
    private JCheckBoxMenuItem jcmiPMCoverCont;


    private JMenu jmForms = null;
    
    private JMenu jmExportForm = null;
    private JMenuItem jmiExportFormFDF = null;
    private JMenuItem jmiExportFormXFDF = null;
    private JMenuItem jmiExportFormXDP = null;
    private JMenuItem jmiExportFormXML = null;
    

    private JMenu jmImportForm = null;
    private JMenuItem jmiImportFormFDF = null;
    private JMenuItem jmiImportFormXFDF = null;
    private JMenuItem jmiImportFormXDP = null;
    
    private JMenuItem jmiResetFields = null;
    
	/**
	 * This method initializes 
	 * 
	 */
	public PDFViewer(String openDoc)
	{
		super();
		initialize(openDoc);
	}

    /**
     * This method initializes jCheckBoxMenuItem	
     * 	
     * @return javax.swing.JCheckBoxMenuItem	
     */
    private JCheckBoxMenuItem getJcbShowSelectToolbar()
    {
        if (jcbShowSelectToolbar == null)
        {
            jcbShowSelectToolbar = new JCheckBoxMenuItem();
            jcbShowSelectToolbar.setText("Show Select Toolbar");
            jcbShowSelectToolbar.setSelected(true);
            jcbShowSelectToolbar.addChangeListener(new javax.swing.event.ChangeListener() { 
                public void stateChanged(javax.swing.event.ChangeEvent e)
                {
                    getPDFViewer().getSelectToolbar().setVisible(jcbShowSelectToolbar.isSelected());
                }
            });
        }
        return jcbShowSelectToolbar;
    }

    public static void  main (final String [] args)
    {
		try
		{
			// Create a log file in the user's home directory
			File outFile = new File(System.getProperty("user.home") + File.separator + "jpdfviewer.log");
			PrintStream out = new PrintStream(new FileOutputStream(outFile));
			System.setOut(out);
			System.setErr(out);
			
			// Initial log information
			SampleUtil.libraryLogStart(PDFViewerBean.class);			
		}
		catch (Throwable t)
		{
			// Do nothing, continue without log file
		}
		
        // Set the look and feel
        setLookAndFeel();
        
		// When rendering, use a substitute font when the specified font cannot
		// be found, or the is an error with the embedded font, otherwise the
		// document will not be rendered and an exception will be thrown.
		FontSettings.setUseSubstituteFont(true);
        
        SwingUtilities.invokeLater(new Runnable() {
        	public void run()
        	{
        		// Set key from command line args
        		if (args != null && args.length > 1)
        		{
        			PDFViewerBean.setWebstartKey(args[1]);
        		}
        		
        		// Load document from command line args
                String loadDoc = null;
                if (args != null && args.length > 0)
                {
                    loadDoc = args [0];
                }

                // Create the viewer frame and show it
                PDFViewer frame = new PDFViewer (loadDoc);
                frame.setVisible(true);
        	}
        });
    }
    
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize(final String loadDoc) 
    {
        this.setJMenuBar(getJMenuBar1());
        this.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
        this.setContentPane(getJPanel());

        DisplayMode dm = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDisplayMode();
        if (dm == null)
        {
            this.setSize((int)(1024 * SampleUtil.getDPIScalingMultiplier()), (int)(768 * SampleUtil.getDPIScalingMultiplier()));
        }
        else
        {
            this.setSize((int)Math.min ((int)(1024 * SampleUtil.getDPIScalingMultiplier()), dm.getWidth() * 0.90), (int)Math.min ((int)(768 * SampleUtil.getDPIScalingMultiplier()), dm.getHeight() * 0.90));
        }
        this.setLocationRelativeTo(null);

        try
		{
    		// Try to set multiple images, java 1.6 only
			Method setIconImages = this.getClass().getMethod("setIconImages", new Class[] {List.class});
			Vector imageList = new Vector();
			imageList.add(ImageIO.read(getClass().getResourceAsStream("jPDFViewer16.png")));
			imageList.add(ImageIO.read(getClass().getResourceAsStream("jPDFViewer32.png")));
			setIconImages.invoke(this, new Object[] {imageList});
		}
		catch (Throwable t1)
		{
			try
	        {
	            this.setIconImage(ImageIO.read(getClass().getResourceAsStream("jPDFViewer16.png")));
	        }
	        catch(Throwable t2)
	        {
	            // ignore
	        }
		}
         
        this.addWindowListener(new java.awt.event.WindowAdapter() { 
        	public void windowOpened(java.awt.event.WindowEvent e) 
        	{    
        	    // Title bar
        	    setTitle("jPDFViewer Sample - " + PDFViewerBean.getVersion());

        	    // Initialize help messages
            	File jarFile = new File ("lib/" + JAR_FILE_NAME);
            	getjlJARLocation().setText (JAR_FILE_NAME + " is located at " + jarFile.getAbsolutePath () + ".");
            	getjlJARLocation().setToolTipText(jarFile.getAbsolutePath ());
            	File samplesDir = new File (SAMPLES_DIR_NAME);
            	getjlSampleLocation().setText ("You can find sample code at " + samplesDir.getAbsolutePath());
            	getjlSampleLocation().setToolTipText(samplesDir.getAbsolutePath());
            	
            	// Load an initial document
                if (loadDoc == null)
                {
                    loadSampleDoc ();
                }
                else
                {
                    loadDocument (loadDoc);
                }

                // Set ourselves up as a drop target to receive PDF files
                setDropTarget(new DropTarget(PDFViewer.this, PDFViewer.this));

                // Optional call to allow keystroke navigation (up/down arrows, page up/down, ...)
                getPDFViewer().getRootPane().getContentPane().requestFocus();
        	}
        });
	}
	
	private void loadSampleDoc ()
	{
	    InputStream inStream = this.getClass().getResourceAsStream ("/blurb.pdf");
	    if (inStream == null)
	    {
	        JOptionPane.showMessageDialog(this, "Unable to find sample pdf.");
	    }
	    else
	    {
		    try
		    {
		        getPDFViewer().loadPDF(inStream);
		    }
		    catch (PDFException pdfE)
		    {
		        JOptionPane.showMessageDialog(this, pdfE.getMessage());
		    }
	    }
	}
	
	private void loadDocument (String loadDoc)
	{
        try
        {
    	    if (loadDoc.startsWith("http:") || loadDoc.startsWith("https:"))
    	    {
    	        getPDFViewer().loadPDF(new URL (loadDoc));
    	    }
    	    else
    	    {
    	        getPDFViewer().loadPDF (loadDoc);
    	    }
        }
        catch (PDFException pdfE)
        {
            JOptionPane.showMessageDialog(this, pdfE.getMessage());
        }
        catch (Throwable t)
        {
            JOptionPane.showMessageDialog(this, "Error opening document: " + t.getMessage());
        }
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getPDFViewer(), BorderLayout.CENTER);
			jPanel.add(getJpAPI(), BorderLayout.SOUTH);
		}
		return jPanel;
	}
	/**
	 * This method initializes PDFViewer	
	 * 	
	 * @return com.qoppa.pdfViewer.PDFViewer	
	 */    
	private PDFViewerBean getPDFViewer()
	{
		if (PDFViewer == null)
		{
			PDFViewer = new PDFViewerBean();

			javax.swing.border.Border empty = javax.swing.BorderFactory.createEmptyBorder(10, 10, 10, 10);
			javax.swing.border.Border line = javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray, 1);
			javax.swing.border.Border border = javax.swing.BorderFactory.createCompoundBorder(empty, line);
			PDFViewer.setBorder(border);

			PDFViewer.getAttachmentPanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
			PDFViewer.getBookmarkPanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
			PDFViewer.getDestinationPanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
			PDFViewer.getLayerPanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
			PDFViewer.getThumbnailPanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
			PDFViewer.getSignaturePanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
			PDFViewer.getCommentPanel().setActivePolicy(PDFPanel.ALWAYS_ACTIVE);
		}
		return PDFViewer;
	}

	/**
	 * This method initializes jpAPI	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJpAPI() {
		if (jpAPI == null) {
			jpAPI = new JPanel();
			jLabel = new JLabel();
			jpAPI.setLayout(new BoxLayout(jpAPI, BoxLayout.Y_AXIS));
			javax.swing.border.Border empty = javax.swing.BorderFactory.createEmptyBorder(0, 10, 10, 10);
			javax.swing.border.Border etched = javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED);
			jpAPI.setBorder(javax.swing.BorderFactory.createCompoundBorder(empty, etched));
			jLabel.setText("to view the API.");
			
			jpAPI.add(Box.createRigidArea(new Dimension(0,4)));
			jpAPI.add(getjlJARLocation());
			jpAPI.add(Box.createRigidArea(new Dimension(0,5)));
			jpAPI.add(getjlSampleLocation());
			jpAPI.add(Box.createRigidArea(new Dimension(0,2)));
			JPanel viewAPI = new JPanel(new FlowLayout(FlowLayout.LEADING));
			viewAPI.add(getJbViewAPI());
			viewAPI.add(jLabel);
			viewAPI.setAlignmentX(Component.LEFT_ALIGNMENT);
			jpAPI.add(viewAPI);
		}
		return jpAPI;
	}
	
	private JLabel getjlSampleLocation()
	{
	    if (jlSampleLocation == null)
	    {
			jlSampleLocation = new JLabel();
			jlSampleLocation.setText("JLabel");
			jlSampleLocation.setAlignmentX(Component.LEFT_ALIGNMENT);
			jlSampleLocation.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 5, 0, 0));
	    }
	    return jlSampleLocation;
	}
	private JLabel getjlJARLocation ()
	{
	    if (jlJARLocation == null)
	    {
			jlJARLocation = new JLabel();
			jlJARLocation.setText("JLabel");
			jlJARLocation.setAlignmentX(Component.LEFT_ALIGNMENT);
			jlJARLocation.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 5, 0, 0));
	    }
	    return jlJARLocation;
	}
	/**
	 * This method initializes jbViewAPI	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbViewAPI() {
		if (jbViewAPI == null) {
			jbViewAPI = new JButton();
			jbViewAPI.setText("Click Here");
			jbViewAPI.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) 
				{    
				    viewAPI();
				}
			});
		}
		return jbViewAPI;
	}
	/**
	 * This method initializes jMenuBar1	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */    
	private JMenuBar getJMenuBar1() {
		if (jMenuBar1 == null) {
			jMenuBar1 = new JMenuBar();
			jMenuBar1.add(getJmFile());
			jMenuBar1.add(getJmEdit());
			jMenuBar1.add(getJmView());
			jMenuBar1.add(getJmToolbar());
			jMenuBar1.add(getJmForms());
			jMenuBar1.add(getJmUserGuide());
		}
		return jMenuBar1;
	}
	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJmFile() {
		if (jmFile == null) {
			jmFile = new JMenu();
			jmFile.setText("File");
			jmFile.add(getjmiOpen());
			jmFile.add(getjmiOpenURL());
			jmFile.add(getjmiPrint());
			jmFile.add(getJmiClose());
			jmFile.add(getJSeparator());
			jmFile.add(getJmiExit());
		}
		return jmFile;
	}
	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJmEdit() {
		if (jmEdit == null) {
			jmEdit = new JMenu();
			jmEdit.setText("Edit");
			jmEdit.add(getJmiSearch());
			jmEdit.add(getJmiAdvSearch());
		}
		return jmEdit;
	}
	/**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJmView() {
		if (jmView == null) {
			jmView = new JMenu();
			jmView.setText("View");
			jmView.add(getJmiTouch());
			jmView.addSeparator();
			jmView.add(getJmiInvertColors());
			jmView.addSeparator();
			jmView.add(getJmiActualSize());
			jmView.add(getJmiFitToWidth());
			jmView.add(getJmiFitToPage());
            jmView.addSeparator();
            jmView.add(getJmiPreviousView());
            jmView.add(getJmiNextView());
            jmView.addSeparator();
            jmView.add(getjmPageMode());
		}
		return jmView;
	}
	/**
	 * This method initializes jMenu1	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJmToolbar() {
		if (jmToolbar == null) {
			jmToolbar = new JMenu();
			jmToolbar.setText("Toolbar");
			jmToolbar.add(getJcbShowToolbar());
			jmToolbar.add(getJcbShowSelectToolbar());
			jmToolbar.add(getJSeparator1());
			jmToolbar.add(getJcbShowOpen());
			jmToolbar.add(getJcbShowPrint());
		}
		return jmToolbar;
	}
	
	/**
     * This method initializes jmForms   
     *  
     * @return javax.swing.JMenu    
     */
    private JMenu getJmForms() {
        if (jmForms == null) {
            jmForms = new JMenu();
            jmForms.setText("Forms");
            jmForms.add(getJmExportForm());
            jmForms.add(getJmImportForm());
            jmForms.addSeparator();
            jmForms.add(getJmiResetFields());
        }
        return jmForms;
    }
    /**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */ 
    private JMenu getJmUserGuide()
	{
		if (jmUserGuide == null)
		{
			jmUserGuide = new JMenu("End-User Guide");
			jmUserGuide.setForeground(Color.blue);
			
			if(!(isSystemWindows() || isSystemMac()))
			{
				jmUserGuide.add(getJmiUserGuide());
			}
			else
			{
				jmUserGuide.addMouseListener(new MouseListener(){
					public void mouseEntered(MouseEvent e)
					{
						PDFViewer.this.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
						e.consume();
					}
	
					public void mouseExited(MouseEvent e)
					{
						PDFViewer.this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						PDFViewer.this.getJmUserGuide().setSelected(false);
					}
	
					public void mousePressed(MouseEvent e)
					{
						openURL(USER_GUIDE, false);
					}

					public void mouseClicked(MouseEvent e)
					{
					}

					public void mouseReleased(MouseEvent e)
					{
					}
				});
			}
		}
		return jmUserGuide;
	}

    /**
	 * This method initializes jMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */ 
	private JMenuItem getJmiUserGuide()
	{
		if(jmiUserGuide == null)
		{
			jmiUserGuide = new JMenuItem("End-User Guide");
			jmiUserGuide.setForeground(Color.blue);
			jmiUserGuide.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) 
				{
					openURL(USER_GUIDE, false);
				}
			});
		}
		return jmiUserGuide;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getjmiOpen() {
		if (jmiOpen == null) {
			jmiOpen = new JMenuItem();
			jmiOpen.setText("Open");
			jmiOpen.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) 
				{
				    File pdfFile = getFile();
				    if (pdfFile != null)
				    {
				        try
				        {
				            getPDFViewer().loadPDF(pdfFile.getAbsolutePath());
				        }
				        catch (PDFException pdfE)
				        {
				            JOptionPane.showMessageDialog(getContentPane(), pdfE.getMessage());
				        }
				    }
				}
			});
		}
		return jmiOpen;
	}

	private JMenuItem getjmiOpenURL() {
		if (jmiOpenURL == null) {
			jmiOpenURL = new JMenuItem();
			jmiOpenURL.setText("Open URL");
			jmiOpenURL.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) 
				{
					String urlString = JOptionPane.showInputDialog(getContentPane(), "Enter URL", "http://");
					if (urlString != null)
					{
						try
						{
							getPDFViewer().loadPDF(new URLPDFSource(new URL(urlString)));
						}
						catch(MalformedURLException badURL)
						{
							badURL.printStackTrace();
							JOptionPane.showMessageDialog(getContentPane(), badURL.getMessage());
						}
						catch (PDFException pdfE)
						{
							pdfE.printStackTrace();
							JOptionPane.showMessageDialog(getContentPane(), pdfE.getMessage());
						}
					}
				}
			});
		}
		return jmiOpenURL;
	}

	/**
	 * This method initializes jMenuItem1	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getjmiPrint() {
		if (jmiPrint == null) {
			jmiPrint = new JMenuItem();
			jmiPrint.setText("Print");
			jmiPrint.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) 
				{
				    try
				    {
				        getPDFViewer().print((PrintSettings)null);
				    }
				    catch (Exception exc)
				    {
				        JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
				    }
				}
			});
		}
		return jmiPrint;
	}
	/**
	 * This method initializes jSeparator	
	 * 	
	 * @return javax.swing.JSeparator	
	 */    
	private JSeparator getJSeparator() {
		if (jSeparator == null) {
			jSeparator = new JPopupMenu.Separator();
		}
		return jSeparator;
	}
	/**
	 * This method initializes jmiClose	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiClose() {
		if (jmiClose == null) {
			jmiClose = new JMenuItem();
			jmiClose.setText("Close");
			jmiClose.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
				    getPDFViewer().clearDocument();
				}
			});
		}
		return jmiClose;
	}
	/**
	 * This method initializes jMenuItem2	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiExit() {
		if (jmiExit == null) {
			jmiExit = new JMenuItem();
			jmiExit.setText("Exit");
			jmiExit.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
				    System.exit(0);
				}
			});
		}
		return jmiExit;
	}
	/**
	 * This method initializes jmiSearch
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJmiSearch() {
		if (jmiSearch == null) {
			jmiSearch = new JMenuItem();
			jmiSearch.setText("Search");
			jmiSearch.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			jmiSearch.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					getPDFViewer().startSearch();
				}
			});
		}
		return jmiSearch;
	}
	/**
	 * This method initializes jmiAdvSearch
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJmiAdvSearch() {
		if (jmiAdvSearch == null) {
			jmiAdvSearch = new JMenuItem();
			jmiAdvSearch.setText("Advanced Search");
			jmiAdvSearch.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.SHIFT_MASK | Toolkit.getDefaultToolkit().getMenuShortcutKeyMask())); 
			jmiAdvSearch.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					getPDFViewer().startAdvancedSearch();
				}
			});
		}
		return jmiAdvSearch;
	}

	/**
	 * This method initializes jmiActualSize
	 * 
	 * @return javax.swing.JMenuItem
	 */
	public JMenuItem getJmiActualSize()
	{
		if (jmiActualSize == null)
		{
			jmiActualSize = new JMenuItem();
			jmiActualSize.setText("Actual Size");
			jmiActualSize.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			jmiActualSize.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setZoomMode(PDFViewerBean.ZOOMMODE_NORMAL);
						getPDFViewer().setScale2D(100);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jmiActualSize;
	}

	/**
	 * This method initializes jmiFitToPage
	 * 
	 * @return javax.swing.JMenuItem
	 */
	public JMenuItem getJmiFitToPage()
	{
		if (jmiFitToPage == null)
		{
			jmiFitToPage = new JMenuItem();
			jmiFitToPage.setText("Fit To Page");
			jmiFitToPage.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_0, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			jmiFitToPage.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setZoomMode(PDFViewerBean.ZOOMMODE_FITPAGE);
						
						getPDFViewer().setPage(getPDFViewer().getPageNumber());
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jmiFitToPage;
	}

	/**
	 * This method initializes jmiFitToWidth
	 * 
	 * @return javax.swing.JMenuItem
	 */
	public JMenuItem getJmiFitToWidth()
	{
		if (jmiFitToWidth == null)
		{
			jmiFitToWidth = new JMenuItem();
			jmiFitToWidth.setText("Fit To Width");
			jmiFitToWidth.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			jmiFitToWidth.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setZoomMode(PDFViewerBean.ZOOMMODE_FITWIDTH);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jmiFitToWidth;
	}

	public JMenuItem getJmiNextView()
	{
		if (jmiNextView == null)
		{
			jmiNextView = new JMenuItem();
			jmiNextView.setText("Next view");
			jmiNextView.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().gotoNextView();
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jmiNextView;
	}

	public JMenuItem getJmiPreviousView()
	{
		if (jmiPreviousView == null)
		{
			jmiPreviousView = new JMenuItem();
			jmiPreviousView.setText("Previous view");
			jmiPreviousView.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().gotoPreviousView();
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jmiPreviousView;
	}

	public JCheckBoxMenuItem getJmiInvertColors()
	{
		if (jcbmiInvertColors == null)
		{
			jcbmiInvertColors = new JCheckBoxMenuItem("Invert Colors");
			jcbmiInvertColors.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						binvert = !binvert;
						getPDFViewer().setInvertColorsMode(binvert);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcbmiInvertColors;
	}

	public JCheckBoxMenuItem getJmiTouch()
	{
		if (jcbmiToggleTouch == null)
		{
			jcbmiToggleTouch = new JCheckBoxMenuItem("Touch Mode");
			jcbmiToggleTouch.setSelected(getPDFViewer().isTouchEnabled());
			jcbmiToggleTouch.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setTouchEnabled(jcbmiToggleTouch.isSelected());
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcbmiToggleTouch;
	}

	public JMenu getjmPageMode()
	{
		if (jmPageMode == null)
		{
			jmPageMode = new JMenu();
			jmPageMode.setText("Page Layout");
			jmPageMode.add(getjcmiPMSingle());
			jmPageMode.add(getjcmiPMContinuous());
			jmPageMode.add(getjcmiPMFacing());
			jmPageMode.add(getjcmiPMFacingCont());
			jmPageMode.add(getjcmiPMCover());
			jmPageMode.add(getjcmiPMCoverCont());
		}
		return jmPageMode;
	}

	public JCheckBoxMenuItem getjcmiPMContinuous()
	{
		if (jcmiPMContinuous == null)
		{
			jcmiPMContinuous = new JCheckBoxMenuItem("Single Continuous");
			getbgPageMode().add(jcmiPMContinuous);
			getbgPageMode().setSelected(jcmiPMContinuous.getModel(), true);
			jcmiPMContinuous.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setPageMode(PDFViewerBean.PAGEMODE_CONTINUOUS);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcmiPMContinuous;
	}

	public JCheckBoxMenuItem getjcmiPMFacing()
	{
		if (jcmiPMFacing == null)
		{
			jcmiPMFacing = new JCheckBoxMenuItem("Facing");
			getbgPageMode().add(jcmiPMFacing);
			jcmiPMFacing.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setPageMode(PDFViewerBean.PAGEMODE_FACING);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcmiPMFacing;
	}

	public JCheckBoxMenuItem getjcmiPMFacingCont()
	{
		if (jcmiPMFacingCont == null)
		{
			jcmiPMFacingCont = new JCheckBoxMenuItem("Facing Continuous");
			getbgPageMode().add(jcmiPMFacingCont);
			jcmiPMFacingCont.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setPageMode(PDFViewerBean.PAGEMODE_FACING_CONTINUOUS);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcmiPMFacingCont;
	}

	public JCheckBoxMenuItem getjcmiPMSingle()
	{
		if (jcmiPMSingle == null)
		{
			jcmiPMSingle = new JCheckBoxMenuItem("Single");
			getbgPageMode().add(jcmiPMSingle);
			jcmiPMSingle.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setPageMode(PDFViewerBean.PAGEMODE_SINGLEPAGE);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcmiPMSingle;
	}

	public JCheckBoxMenuItem getjcmiPMCover()
	{
		if (jcmiPMCover == null)
		{
			jcmiPMCover = new JCheckBoxMenuItem("Cover");
			getbgPageMode().add(jcmiPMCover);
			jcmiPMCover.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setPageMode(PDFViewerBean.PAGEMODE_COVER);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcmiPMCover;
	}

	public JCheckBoxMenuItem getjcmiPMCoverCont()
	{
		if (jcmiPMCoverCont == null)
		{
			jcmiPMCoverCont = new JCheckBoxMenuItem("Cover Continuous");
			getbgPageMode().add(jcmiPMCoverCont);
			jcmiPMCoverCont.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					try
					{
						getPDFViewer().setPageMode(PDFViewerBean.PAGEMODE_COVER_CONTINUOUS);
					}
					catch (Exception exc)
					{
						JOptionPane.showMessageDialog(getContentPane(), exc.getMessage());
					}
				}
			});
		}
		return jcmiPMCoverCont;
	}

	public ButtonGroup getbgPageMode()
	{
		if (bgPageMode == null)
		{
			bgPageMode = new ButtonGroup();
		}
		return bgPageMode;
	}
	/**
	 * This method initializes jCheckBoxMenuItem	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	 */    
	private JCheckBoxMenuItem getJcbShowToolbar() {
		if (jcbShowToolbar == null) {
			jcbShowToolbar = new JCheckBoxMenuItem();
			jcbShowToolbar.setText("Show Toolbar");
			jcbShowToolbar.setSelected(true);
			jcbShowToolbar.addChangeListener(new javax.swing.event.ChangeListener() { 
				public void stateChanged(javax.swing.event.ChangeEvent e)
				{
				    getPDFViewer().getToolbar().setVisible(jcbShowToolbar.isSelected());
				}
			});
		}
		return jcbShowToolbar;
	}
	/**
	 * This method initializes jSeparator1	
	 * 	
	 * @return javax.swing.JSeparator	
	 */    
	private JSeparator getJSeparator1() {
		if (jSeparator1 == null) {
			jSeparator1 = new JPopupMenu.Separator();
		}
		return jSeparator1;
	}
	/**
	 * This method initializes jCheckBoxMenuItem1	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	 */    
	private JCheckBoxMenuItem getJcbShowOpen() {
		if (jcbShowOpen == null) {
			jcbShowOpen = new JCheckBoxMenuItem();
			jcbShowOpen.setText("Show Open Button");
			jcbShowOpen.setSelected(true);
			jcbShowOpen.addChangeListener(new javax.swing.event.ChangeListener() { 
				public void stateChanged(javax.swing.event.ChangeEvent e)
				{
				    getPDFViewer().getToolbar().getjbOpen().setVisible (jcbShowOpen.isSelected());
				}
			});
		}
		return jcbShowOpen;
	}
	/**
	 * This method initializes jCheckBoxMenuItem2	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	 */    
	private JCheckBoxMenuItem getJcbShowPrint() {
		if (jcbShowPrint == null) {
			jcbShowPrint = new JCheckBoxMenuItem();
			jcbShowPrint.setText("Show Print Button");
			jcbShowPrint.setSelected(true);
			jcbShowPrint.addItemListener(new java.awt.event.ItemListener() { 
				public void itemStateChanged(java.awt.event.ItemEvent e) 
				{
				    getPDFViewer().getToolbar().getjbPrint().setVisible (jcbShowPrint.isSelected());
				}
			});
		}
		return jcbShowPrint;
	}
    
   	private boolean isSystemMac() 
   	{
   		// Check the OS
   		String osName = System.getProperty ("os.name");
   		osName = osName.toLowerCase();
   		int firstIndexOfMac = osName.indexOf (OS_MAC);
   		if (firstIndexOfMac == -1)
   		{
   			return false;
   		}
   		else
   		{
   			return true;
   		}
   	}
   	/**
   	 * Insert the method's description here.
   	 * Creation date: (11/10/2003 9:16:17 PM)
   	 * @return boolean
   	 */
   	private boolean isSystemWindows()
   	{
   		// Check the OS
   		String osName = System.getProperty ("os.name");
   		osName = osName.toLowerCase();
   		return osName.startsWith(OS_WINDOWS_START);
   	}
    
    /**
	 * This method displays a file chooser dialog and gets a file name.	
	 * 	
	 * @return java.io.File	
	 */ 
	private File getFile (boolean open, String fileFilter, String defaultFileName)
	{
	    // Get first file name
	    JFileChooser fileChooser = new JFileChooser ();

	    // set default file name
        if(defaultFileName != null)
        {
            fileChooser.setSelectedFile(new File(defaultFileName));
        }
        
	    // filter files
	    fileChooser.setFileFilter(new CustomFileFilter(fileFilter));
	    
        // Show the open dialog
		if(open)
		{
		    if (fileChooser.showOpenDialog(this) == JFileChooser.CANCEL_OPTION)
		    {
		        return null;
		    }
		}
		// Show save dialog
		else
		{
			if (fileChooser.showSaveDialog(this) == JFileChooser.CANCEL_OPTION)
		    {
		        return null;
		    }
		}
		
		// Save last directory location
		File chosenFile = fileChooser.getSelectedFile();
		return chosenFile;
	}
    
	private void viewAPI()
	{
		openURL(new File (API_INDEX_FILENAME).getAbsolutePath(), true);
	}
	
   	private void openURL(String open, boolean isFile)
   	{
   		try
   		{
   			if (isSystemWindows())
   			{
   				if (isFile)
   				{
   					Runtime.getRuntime().exec ("rundll32 url.dll,FileProtocolHandler " + open);
   				}
   				else
   				{
   				    Runtime.getRuntime().exec("cmd.exe /c start " + open);
   				}
   			}
   			else if (isSystemMac())
   			{
   				String [] cmdArray = new String [2];
   				cmdArray [0] = "open";
   				cmdArray [1] = open;
   				Runtime.getRuntime ().exec (cmdArray);
   			}
   			// assume Unix or Linux
   	        else 
   	        { 
   	            String[] browsers = {"firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape", "google-chrome" };
   	            String browser = null;
   	            for (int count = 0; count < browsers.length && browser == null; count++)
   	            {
   	               if (Runtime.getRuntime().exec(new String[] {"which", browsers[count]}).waitFor() == 0)
   	               {
   	                  browser = browsers[count];
   	                  break;
   	               }
   	            }
   	            if (browser == null)
   	               throw new Exception("Could not find web browser");
   	            else
   	               Runtime.getRuntime().exec(new String[] {browser, open});
   	        }
   		}
   		catch (Throwable t)
   		{
   			javax.swing.JOptionPane.showMessageDialog (this, t.getMessage ());
   		}
   	}

   	/**
	 * This method shows a JFileChooser with a PDF filter
	 * and returns the chosen file.  If the user hits cancel,
	 * the method returns null.
	 * 	
	 * @param	currentFile	The file chooser will select this file
	 * 			when it comes up.
	 * 
	 * @return	Chosen file.	
	 */    
	private File getFile()
	{
	    // Create file chooser
		JFileChooser fileChooser = new JFileChooser ();
		
		// PDF File filter
		PDFFileFilter pdfFilter = new PDFFileFilter();

		// Set the filter
		fileChooser.setFileFilter(pdfFilter);

		// Show the JFileChooser
		int rc = fileChooser.showOpenDialog(this);
		if (rc == JFileChooser.APPROVE_OPTION)
		{
		    return fileChooser.getSelectedFile();
		}
		else
		{
			return null;
		}
	}
    /**
	 * This method initializes qmExportForm   
	 *  
	 * @return javax.swing.JMenu    
	 */    
	public JMenu getJmExportForm() {
	    if (jmExportForm == null) {
	    	jmExportForm = new JMenu();
	    	jmExportForm.setText("Export Form");
	    	jmExportForm.add(getJmiExportFormFDF());
	    	jmExportForm.add(getJmiExportFormXFDF());
	    	jmExportForm.add(getJmiExportFormXDP());
	    	jmExportForm.add(getJmiExportFormXML());
	    }
	    return jmExportForm;
	}

	/**
	 * This method initializes jmiExportFormFDF
	 *  
	 * @return javax.swing.JMenuItem   
	 */    
	public JMenuItem getJmiExportFormFDF() {
	    if (jmiExportFormFDF == null) {
	        jmiExportFormFDF = new JMenuItem();
	        jmiExportFormFDF.setText("FDF");
	        jmiExportFormFDF.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) 
				{    
					if(getPDFViewer().getAcroForm() != null)
					{
						File file = getFile(false, "FDF", "out.fdf");
						if(file != null)
						{
							try
							{
								getPDFViewer().getAcroForm().exportAsFDF(file.getAbsolutePath(), true);
							}
							catch(PDFException pdfException)
						    {
						       JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
						    }
							catch (IOException ioe)
							{
							    JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + ioe.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
			});
	    }
	    return jmiExportFormFDF;
	}

	/**
	 * This method initializes jmiExportFormXDP	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	public JMenuItem getJmiExportFormXDP()
	{
	    if (jmiExportFormXDP == null)
	    {
	        jmiExportFormXDP = new JMenuItem();
	        jmiExportFormXDP.setText("XDP");
	        jmiExportFormXDP.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{    
					if(getPDFViewer().getAcroForm() != null)
					{
				        File file = getFile(false, "XDP", "out.xdp");
			            if(file != null)
			            {
			                try
			                {
			                	getPDFViewer().getAcroForm().exportAsXDP(file.getAbsolutePath(), true);
			                }
			                catch(PDFException pdfException)
			                {
			                   JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as xdp " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			                }
			                catch(IOException ioException)
			                {
			                   JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as xdp " + ioException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			                }
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
			            }
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
	        });
	    }
	    return jmiExportFormXDP;
	}

	/**
	 * This method initializes jmiExportFormXFDF
	 *  
	 * @return javax.swing.JMenuItem   
	 */    
	public JMenuItem getJmiExportFormXFDF() {
	    if (jmiExportFormXFDF == null) {
	        jmiExportFormXFDF = new JMenuItem();
	        jmiExportFormXFDF.setText("XFDF");
	        jmiExportFormXFDF.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{    
					if(getPDFViewer().getAcroForm() != null)
					{
				        File file = getFile(false, "XFDF", "out.xfdf");
			            if(file != null)
			            {
			                try
			                {
			                	getPDFViewer().getAcroForm().exportAsXFDF(file.getAbsolutePath(), true);
			                }
			                catch(PDFException pdfException)
			                {
			                   JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as xfdf " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			                }
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
			            }
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
	        });
	    }
	    return jmiExportFormXFDF;
	}

	/**
	 * This method initializes jmiExportFormXML
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	public JMenuItem getJmiExportFormXML()
	{
	    if (jmiExportFormXML == null)
	    {
	        jmiExportFormXML = new JMenuItem();
	        jmiExportFormXML.setText("XML"); 
	        jmiExportFormXML.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{    
					if(getPDFViewer().getAcroForm() != null)
					{
				        File file = getFile(false, "XML", "out.xml");
			            if(file != null)
			            {
			                try
			                {
			                	getPDFViewer().getAcroForm().exportAsXFDF(file.getAbsolutePath(), true);
			                }
			                catch(PDFException pdfException)
			                {
			                   JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as xml " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			                }
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
			            }
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
	        });
	    }
	    return jmiExportFormXML;
	}

	/**
	 * This method initializes jmiImportFormFDF
	 *  
	 * @return javax.swing.JMenuItem   
	 */    
	public JMenuItem getJmiImportFormFDF() {
	    if (jmiImportFormFDF == null) {
	        jmiImportFormFDF = new JMenuItem();
	        jmiImportFormFDF.setText("FDF");
	        jmiImportFormFDF.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{ 
	        		if(getPDFViewer().getAcroForm() != null)
					{
	        			File file = getFile(true, "FDF", null);
						if(file != null)
						{
							try
							{
								getPDFViewer().getAcroForm().importFDF(file.getAbsolutePath());
								// message
			                    JOptionPane.showMessageDialog(getPDFViewer(), "FDF file imported", "Message", JOptionPane.INFORMATION_MESSAGE);
			                }
							catch(PDFException pdfException)
						    {
						       JOptionPane.showMessageDialog(getPDFViewer(), "Error importing from FDF " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
						    }
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
	        });
	    }
	    return jmiImportFormFDF;
	}

	/**
	 * This method initializes jmiImportFormXDP	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	public JMenuItem getJmiImportFormXDP()
	{
	    if (jmiImportFormXDP == null)
	    {
	        jmiImportFormXDP = new JMenuItem();
	        jmiImportFormXDP.setText("XDP");
	        jmiImportFormXDP.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{ 
	        		if(getPDFViewer().getAcroForm() != null)
					{
	        			File file = getFile(true, "XDP", null);
						if(file != null)
						{
							try
							{
								getPDFViewer().getAcroForm().importXDP(file.getAbsolutePath());
								// message
			                    JOptionPane.showMessageDialog(getPDFViewer(), "XDP file imported", "Message", JOptionPane.INFORMATION_MESSAGE);
			                }
							catch(PDFException pdfException)
						    {
						       JOptionPane.showMessageDialog(getPDFViewer(), "Error importing from XDP " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
						    }
							catch(IOException ioException)
						    {
						       JOptionPane.showMessageDialog(getPDFViewer(), "Error importing from FDF " + ioException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
						    }
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
	        });
	    }
	    return jmiImportFormXDP;
	}

	/**
	 * This method initializes jmiImportFormXFDF
	 *  
	 * @return javax.swing.JMenuItem   
	 */    
	public JMenuItem getJmiImportFormXFDF() {
	    if (jmiImportFormXFDF == null) {
	        jmiImportFormXFDF = new JMenuItem();
	        jmiImportFormXFDF.setText("XFDF");
	        jmiImportFormXFDF.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{ 
	        		if(getPDFViewer().getAcroForm() != null)
					{
	        			File file = getFile(true, "XFDF", null);
						if(file != null)
						{
							try
							{
								getPDFViewer().getAcroForm().importXFDF(file.getAbsolutePath());
								// message
			                    JOptionPane.showMessageDialog(getPDFViewer(), "XFDF file imported", "Message", JOptionPane.INFORMATION_MESSAGE);
			                }
							catch(PDFException pdfException)
						    {
						       JOptionPane.showMessageDialog(getPDFViewer(), "Error importing from XFDF " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
						    }
							catch (Throwable t)
							{
								 JOptionPane.showMessageDialog(getPDFViewer(), "Error exporting as fdf " + t.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
				}
	        });
	    }
	    return jmiImportFormXFDF;
	}

	/**
	 * This method initializes jmiImportForm   
	 *  
	 * @return javax.swing.JMenu    
	 */    
	public JMenu getJmImportForm() {
	    if (jmImportForm == null) {
	    	jmImportForm = new JMenu();
	    	jmImportForm.setText("Import Form");
	    	jmImportForm.add(getJmiImportFormFDF());
	    	jmImportForm.add(getJmiImportFormXFDF());
	    	jmImportForm.add(getJmiImportFormXDP());
	    }
	    return jmImportForm;
	}

	/**
	 * This method initializes jmiResetFields   
	 *  
	 * @return com.qoppa.pdfStudio.components.JMenuItem 
	 */    
	public JMenuItem getJmiResetFields() {
	    if (jmiResetFields == null) {
	        jmiResetFields = new JMenuItem();
	        jmiResetFields.setText("Reset Fields");
	        jmiResetFields.addActionListener(new java.awt.event.ActionListener() { 
	        	public void actionPerformed(java.awt.event.ActionEvent e) 
				{ 
	        		if(getPDFViewer().getAcroForm() != null)
					{
				        try
			            {
				        	getPDFViewer().getAcroForm().resetFields();
			
			                // message
			                JOptionPane.showMessageDialog(getPDFViewer(), "Fields reset", "Message", JOptionPane.INFORMATION_MESSAGE);
			            }
			            catch(PDFException pdfException)
			            {
			               JOptionPane.showMessageDialog(getPDFViewer(), "Error resetting fields " + pdfException.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			            }
					}
					else
					{
						JOptionPane.showMessageDialog(getPDFViewer(), "No form data available", "Warning", JOptionPane.WARNING_MESSAGE);
				    }
		        }
	        });
	    }
	    return jmiResetFields;
	}

	/**
     * Set the look and feel.  The jPDFViewer library is look and feel independent, so it
     * will inherit any look and feel set by the host application.  In this sample, we are
     * setting the look and feel to the system default look and feel.
     */
    private static void setLookAndFeel ()
    {
        // Set the look and feel
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            if (UIManager.getSystemLookAndFeelClassName() != null && UIManager.getSystemLookAndFeelClassName().toLowerCase().indexOf("windows") != -1)
            {
            	UIManager.put("TextArea.font", UIManager.get("TextField.font"));
            }
        }
        catch (Throwable t)
        {
            try
            {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
            catch (Throwable tt)
            {
                // ignore
            }
        }
    }
    
    //
    // Implementation of drop target methods
    //
    public void dragEnter(DropTargetDragEvent dtde){}
    public void dragExit(DropTargetEvent dte) {}
    public void dragOver(DropTargetDragEvent dtde) {}
    public void dropActionChanged(DropTargetDragEvent dtde) {}

    public void drop(java.awt.dnd.DropTargetDropEvent dtde) 
    {
        // Get the action and accept the drop
        int action = dtde.getDropAction();
        dtde.acceptDrop(action);

        // Get the transferable
        Transferable t = dtde.getTransferable();
        if (t != null)
        {
            try
            {
            	List list = null; 
            	DataFlavor uriListFlavor = new DataFlavor("text/uri-list;class=java.lang.String");
            	
                // We can take files as dropped objects
                if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) 
                {
                    // Get the list of files
                    list = (List) t.getTransferData(DataFlavor.javaFileListFlavor); 
                }
				else if (t.isDataFlavorSupported(uriListFlavor))
				{
					// Parse the list of files
					list = textURIListToFileList((String) t.getTransferData(uriListFlavor));
				}
                
                if (list != null && list.size() > 0)
                {
                    File file = (File) list.get(0);
                    if (file.getName() != null && file.getName().toLowerCase().endsWith(".pdf"))
                    {
                    	// Load the PDF
                        getPDFViewer().loadPDF(file.getAbsolutePath());
                    }
                }
            }
            catch (Throwable te)
            {
                JOptionPane.showMessageDialog(this, te.getMessage());
            }
        }
    }
    
	private List textURIListToFileList(String data)
	{
		List list = new Vector();
		for (StringTokenizer st = new StringTokenizer(data, "\r\n"); st.hasMoreTokens(); )
		{
			String s = st.nextToken();
			if (s.startsWith("#"))
			{
				continue;
			}
			try
			{
				URI uri = new URI(s);
				File file = new File(uri);
				list.add(file);
			}
			catch (java.net.URISyntaxException e)
			{
			}
			catch (IllegalArgumentException e)
			{
			}
		}
		return list;
	}
	
}
