package jPDFViewerSamples.history;

import jPDFViewerSamples.SampleUtil;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import com.qoppa.pdfViewer.PDFViewerBean;
import com.qoppa.pdfViewer.history.HistoryListener;

/**
 * This sample adds Next/Prev buttons using a HistoryListener and the document's LocationHistory.
 *
 */
public class SimpleDocumentHistory implements HistoryListener
{
	private PDFViewerBean m_ViewerBean;
	private JButton m_NextViewButton;
	private JButton m_PrevViewButton;
		
	public SimpleDocumentHistory()
	{
		init();
	}
	
	private void init()
	{		
		m_ViewerBean = new PDFViewerBean();
		m_ViewerBean.getToolbar().add(getPrevViewButton());
		m_ViewerBean.getToolbar().add(getNextViewButton());
		m_ViewerBean.addHistoryListener(this);
		
		getPrevViewButton().addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e)
			{
				m_ViewerBean.gotoPreviousView();
			}
		});
		
		getNextViewButton().addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e)
			{
				m_ViewerBean.gotoNextView();
			}
		});
		
		toggleButtons(m_ViewerBean);
	}
	
	/**
	 * Displays the PDFViewerBean in a new JFrame.
	 */
	public void showViewerBean()
	{
		JFrame frame = new JFrame();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setContentPane(m_ViewerBean);
		frame.setSize((int)(1000 * SampleUtil.getDPIScalingMultiplier()), (int)(600 * SampleUtil.getDPIScalingMultiplier()));
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}
	
	private JButton getPrevViewButton()
	{
		if (m_PrevViewButton == null)
		{
			m_PrevViewButton = new JButton("<< Prev");
		}
		return m_PrevViewButton;
	}

	private JButton getNextViewButton()
	{
		if (m_NextViewButton == null)
		{
			m_NextViewButton = new JButton("Next >>");
		}
		return m_NextViewButton;
	}
	
	public void previousDocument(PDFViewerBean viewerBean)
	{
	}

	public void nextDocument(PDFViewerBean viewerBean)
	{
	}

	public void beforeDocumentSet(PDFViewerBean viewerBean)
	{	
	}
	
	public void afterDocumentSet(PDFViewerBean viewerBean)
	{		
		// Update the navigation history buttons
		toggleButtons(viewerBean);
	}
	
	public void locationChanged(PDFViewerBean viewerBean, boolean newLocation)
	{
		toggleButtons(viewerBean);
	}
	
	// Toggle the next and previous history buttons
	private void toggleButtons(PDFViewerBean viewerBean)
	{
		if (viewerBean.getDocument() == null)
		{
			getNextViewButton().setEnabled(false);
			getPrevViewButton().setEnabled(false);
		}
		else
		{
			getNextViewButton().setEnabled(viewerBean.getLocationHistory().hasNext());
			getPrevViewButton().setEnabled(viewerBean.getLocationHistory().hasPrevious());
		}
	}
	
	public static void main(String [] args)
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run()
			{
				SimpleDocumentHistory sample = new SimpleDocumentHistory();
				sample.showViewerBean();
			}
		});
	}
}
