/*
 * Created on Jul 3, 2009
 *
 */
package jPDFViewerSamples.textSearch;

import jPDFViewerSamples.SampleUtil;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import com.qoppa.pdf.TextPositionWithContext;

public class TSCellRenderer extends JComponent implements ListCellRenderer
{
    private Font m_PlainFont;
    private Font m_BoldFont;

    private Color m_Background;
    private Color m_Foreground;
    private Dimension m_PreferredSize = new Dimension(0, (int)(15 * SampleUtil.getDPIScalingMultiplier()));

    private String m_BeforeText;
    private String m_SearchText;
    private String m_AfterText;

    private final static int MARGIN_TOP = 1;
    private final static int MARGIN_BOTTOM = 1;
    private final static int MARGIN_LEFT = 2;
        
    public TSCellRenderer (Font plainFont, Font boldFont)
    {
        m_PlainFont = plainFont;
        m_BoldFont = boldFont;
    }

    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
    {
        TextPositionWithContext to = (TextPositionWithContext)value;
        m_BeforeText = to.getBeforeText();
        m_SearchText = to.getText();
        m_AfterText = to.getAfterText();
            
        if (isSelected)
        {
            m_Background = list.getSelectionBackground();
            m_Foreground = list.getSelectionForeground();
        }
        else
        {
            m_Background = null;
            m_Foreground = list.getForeground();
        }
            
        FontMetrics fmPlain = getFontMetrics(m_PlainFont);
        FontMetrics fmBold = getFontMetrics(m_BoldFont);

        m_PreferredSize.height = fmBold.getLeading() + fmBold.getAscent() + fmBold.getDescent() + MARGIN_TOP + MARGIN_BOTTOM;
        m_PreferredSize.width = fmPlain.stringWidth(m_BeforeText) + fmBold.stringWidth(m_SearchText) + fmPlain.stringWidth(m_AfterText);
            
        return this;
    }
        
    public Dimension getPreferredSize()
    {
        return m_PreferredSize;
    }
        
    public void paint(Graphics g)
    {
        // Set background and foreground colors
        if (m_Background != null)
        {
            g.setColor(m_Background);
            g.fillRect(0, 0, getWidth(), getHeight());
        }
        g.setColor (m_Foreground);
            
        // Draw the string
        g.setFont (m_PlainFont);
        int yOffset = g.getFontMetrics().getAscent() + MARGIN_TOP;
        int xPos = MARGIN_LEFT;
            
        g.drawString (m_BeforeText, xPos, yOffset);
        xPos += g.getFontMetrics().stringWidth(m_BeforeText);
            
        g.setFont(m_BoldFont);
        g.drawString(m_SearchText, xPos, yOffset);
        xPos += g.getFontMetrics().stringWidth(m_SearchText);

        g.setFont (m_PlainFont);
        g.drawString(m_AfterText, xPos, yOffset);
    }
}
