/*
 * Created on May 22, 2009
 *
 */
package jPDFViewerSamples.textSearch;

import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Point2D;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.qoppa.pdf.PDFException;
import com.qoppa.pdf.TextPositionWithContext;
import com.qoppa.pdf.dom.IPDFDocument;
import com.qoppa.pdfViewer.IPDFPageView;
import com.qoppa.pdfViewer.PDFViewerBean;

public class TextSearchController implements ActionListener, KeyListener, ListSelectionListener
{
    protected TextSearchDialog m_Dialog;
    private IPDFDocument m_Document;
    private PDFViewerBean m_Bean;
    
    private final static String CMD_CLOSE = "Close";
    private final static String CMD_SEARCH = "Search";
    private final static String CMD_NEXT = "Next";
    private final static String CMD_PREVIOUS = "Previous";
    
    private AbstractAction m_NextAction = new AbstractAction (CMD_NEXT)
    {
        public void actionPerformed(ActionEvent e)
        {
            doNext();
        }
    };
    
    private AbstractAction m_PreviousAction = new AbstractAction (CMD_PREVIOUS)
    {
        public void actionPerformed(ActionEvent e)
        {
            doPrevious();
        }
    };
    
    private AbstractAction m_CloseDialog = new AbstractAction ()
    {
        public void actionPerformed(ActionEvent e)
        {
            doClose();
        }
    };
    
    public TextSearchController (Window parent, PDFViewerBean bean)
    {
            // Create and center the dialog
            if(m_Dialog == null)
            {
                m_Dialog = TextSearchDialog.getInstance(parent);
            }
            
            // dialog at the top left corner of the parent window
            m_Dialog.setLocation (parent.getX () + 40, parent.getY() + 40);
            
            // Create text search cell renderer
            Font plainFont = m_Dialog.getJlOccurences().getFont();
            Font boldFont = plainFont;
            if (plainFont.isBold())
            {
                plainFont = plainFont.deriveFont (Font.PLAIN);
            }
            else
            {
                boldFont = plainFont.deriveFont (Font.BOLD);
            }
            m_Dialog.getJlOccurences().setCellRenderer(new TSCellRenderer(plainFont, boldFont));
            
            // listen to keys F3 (next occurence) and SHIFT F3 (previous occurence)
            InputMap inMap = m_Dialog.getJpContentPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
            inMap.put (KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0), CMD_NEXT);
            m_Dialog.getJpContentPane().getActionMap().put(CMD_NEXT, m_NextAction);
            inMap.put (KeyStroke.getKeyStroke(KeyEvent.VK_F3, InputEvent.SHIFT_MASK), CMD_PREVIOUS);
            m_Dialog.getJpContentPane().getActionMap().put(CMD_PREVIOUS, m_PreviousAction);
            inMap.put (KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), CMD_CLOSE);
            m_Dialog.getJpContentPane().getActionMap().put(CMD_CLOSE, m_CloseDialog);

            m_Bean = bean;
            
            // Listeners
            m_Dialog.getJbClose().setActionCommand(CMD_CLOSE);
            m_Dialog.getJbClose().addActionListener(this);
            m_Dialog.getjbSearch().setActionCommand(CMD_SEARCH);
            m_Dialog.getjbSearch().addActionListener(this);
            m_Dialog.getjtSearchText().addKeyListener(this);
            m_Dialog.getJlOccurences().addKeyListener(this);
            m_Dialog.getJlOccurences().addListSelectionListener(this);
            m_Dialog.getjbNext().setActionCommand(CMD_NEXT);
            m_Dialog.getjbNext().addActionListener(this);
            m_Dialog.getjbPrevious().setActionCommand(CMD_PREVIOUS);
            m_Dialog.getjbPrevious().addActionListener(this);
    }
    
    public void showDialog(IPDFDocument doc)
    {
        // save document
        m_Document = doc;
        
        // select text in the search box
        m_Dialog.getjtSearchText().selectAll();
        
        // show the dialog
        m_Dialog.setVisible(true);
        
        return;
    }
    
    protected void addOccurences(Vector occurences)
    {
        // Initialize the table
        if(occurences != null)
        {
            DefaultListModel model = (DefaultListModel) m_Dialog.getJlOccurences().getModel();
            for (int count = 0; count < occurences.size(); ++count)
            {
                model.addElement(occurences.get(count));
            }
        }
    }
    
    public void doClose()
    {
        // dispose the dialog
        m_Dialog.dispose();
    }
    
    public void clear()
    {
        DefaultListModel listModel = new DefaultListModel ();
        m_Dialog.getJlOccurences().setModel(listModel);
        m_Dialog.getjlNumberOccurences().setText ("<html> <b> 0 Occurences" + "</b></hmtl>");
    }
    
    public void doNext()
    {
        final int currentIndex = m_Dialog.getJlOccurences().getSelectedIndex();
        int nextIndex = currentIndex + 1;
        if(m_Dialog.getJlOccurences().getModel() != null && m_Dialog.getJlOccurences().getModel().getSize() > 0
                && currentIndex + 1 < m_Dialog.getJlOccurences().getModel().getSize())
        {
            m_Dialog.getJlOccurences().requestFocus();
            m_Dialog.getJlOccurences().setSelectedIndex(nextIndex);
            repaintRow(currentIndex);
            m_Dialog.getJlOccurences().ensureIndexIsVisible(nextIndex);
        }
    }
    
    private void repaintRow(final int index)
    {
        if(index >= 0 && index < m_Dialog.getJlOccurences().getModel().getSize())
        {
            SwingUtilities.invokeLater(new Runnable ()
            {
                public void run() 
                {
                    m_Dialog.getJlOccurences().repaint(m_Dialog.getJlOccurences().getCellBounds(index, index));
                }
            });
            
        }
    }
    
    public void doPrevious()
    {
        final int currentIndex = m_Dialog.getJlOccurences().getSelectedIndex();
        int previousIndex = currentIndex - 1;
        if(m_Dialog.getJlOccurences().getModel() != null && m_Dialog.getJlOccurences().getModel().getSize() > 0
                && currentIndex - 1 >= 0)
        {
            m_Dialog.getJlOccurences().requestFocus();
            m_Dialog.getJlOccurences().setSelectedIndex(previousIndex);
            repaintRow(currentIndex);
            m_Dialog.getJlOccurences().ensureIndexIsVisible(previousIndex);
        }
    }
    
    public void doSearch() 
    {
        // reset list
        clear();
        
        // start a new thread to search
        Thread processThread = new Thread ()
        {
            public void run() 
            {
                search();
            }
        };
        processThread.start();
    }
    
    
    protected void search()
    {
        if(m_Document != null)
        {
            // search options
            boolean caseSensitive = m_Dialog.getJcbCaseSensitive().isSelected();
            boolean wholeWords = m_Dialog.getJcbWholeWord().isSelected();
            
            try
            {
                // empty search
                String searchText = m_Dialog.getjtSearchText().getText();
                if(isEmptyString(searchText) || isEmptyString(searchText.trim()))
                {
                    JOptionPane.showMessageDialog(m_Dialog, "Empty Search");
                    return;
                }
                
                Vector allOccurences = new Vector();
                Vector pageOccurences = new Vector();
                int totalOccurences = 0;

                // look for text occurrences on all pages
                for(int pageIx = 0; pageIx < m_Document.getPageCount(); pageIx++)
                {
                    // update progress label
                    setProgress((pageIx + 1), pageOccurences, totalOccurences);
                    
                    pageOccurences = m_Document.getIPage(pageIx).findTextWithContext(searchText, caseSensitive, wholeWords);
                    allOccurences.addAll(pageOccurences);
                    totalOccurences = allOccurences.size();
                }
            
                // final progress
                setProgress(-1, pageOccurences, totalOccurences);
                
                // select first row
                doNext();
            }
            catch(PDFException pe)
            {
                JOptionPane.showMessageDialog(m_Dialog, pe.getMessage());
            }
        }
    }
    
    
    public void actionPerformed (ActionEvent ae)
    {
        if (ae.getActionCommand() == CMD_CLOSE)
        {
            doClose();
        }
        else if (ae.getActionCommand() == CMD_SEARCH)
        {
            doSearch();
        }
        else if (ae.getActionCommand() == CMD_NEXT)
        {
            doNext();
        }
        else if (ae.getActionCommand() == CMD_PREVIOUS)
        {
            doPrevious();
        }
    }

    public void keyTyped(KeyEvent arg0)
    {
       
    }

    public void keyPressed(KeyEvent e)
    {
        if(e.getKeyCode() == KeyEvent.VK_ENTER)
        {
            doSearch();
        }
    }

    public void keyReleased(KeyEvent arg0)
    {
        // nothing to do 
    }
    
    private void setProgress (int pageNumber, final Vector pageOccurences, int totalOccurences)
    {
        String message = "";
        if(pageNumber >= 1)
        {
            message = "<html><b>... Page " + pageNumber + " ...</b></html>";
        }
        final String finalMsg = message;
        final String lblOccurences = "<html><b>" + totalOccurences + " Occurences" + "</b></html>" ;
        if (SwingUtilities.isEventDispatchThread())
        {
            m_Dialog.getjlProgress().setText (finalMsg);
            m_Dialog.getjlNumberOccurences().setText (lblOccurences);
            //add page occurences to the list
            addOccurences(pageOccurences);
        }
        else
        {
            try
            {
                SwingUtilities.invokeAndWait(new Runnable () 
                {
                    public void run ()
                    {
                        m_Dialog.getjlProgress().setText (finalMsg);
                        m_Dialog.getjlNumberOccurences().setText (lblOccurences);
                        // add page occurrences to the list
                        addOccurences(pageOccurences);
                    }
                });
            }
            catch (Exception e)
            {}
        }
    }
    
    private void setTextSelection (Object [] selectList)
    {
        // Clear current selection
        clearSelection();

        // Loop through occurrences
        int currentPage = -1;
        int numSelections = 0;
        MultiTextSelection pageSelection = new MultiTextSelection();
        for (int index = 0; selectList != null && index < selectList.length; ++index)
        {
            // Number of selections
            ++numSelections;

            // Get the next text occurrence
            TextPositionWithContext tpc = (TextPositionWithContext)selectList [index];
            
            // Get the selection page number
            int selectPageNumber = tpc.getPageNumber();
            if (currentPage != selectPageNumber)
            {
                // Add selections to previous page
                if (pageSelection.getSelectionCount() > 0)
                {
                    // set text selected
                    ((IPDFPageView)m_Bean.getPageView(currentPage)).setTextSelection(pageSelection);
                    
                    // repaint the page
                    m_Bean.getPageView(currentPage).repaint();
                }
                
                // Create new text selection
                pageSelection = new MultiTextSelection();
                currentPage = selectPageNumber;
            }
            
            // Add selected text to the selection list
            pageSelection.addSelection(tpc);
        }
        
        // Add selections to the last page
        if (pageSelection.getSelectionCount() > 0)
        {
            // set text selected
            ((IPDFPageView)m_Bean.getPageView(currentPage)).setTextSelection(pageSelection);
            
            // repaint the page
            m_Bean.getPageView(currentPage).repaint();
        }

        // show the selection
        if (numSelections == 1)
        {
            // Get the first text occurrence
            TextPositionWithContext tpc = (TextPositionWithContext)selectList [0];

            // go to the page and scroll to the selected text if needed, centering it
            Point2D [] quad = tpc.getViewQuadrilateral();
            m_Bean.scrollToPage (tpc.getPageNumber(), (int) quad[0].getX(),  (int) quad[0].getY(), true);
        }
    }

    private void clearSelection ()
    {
        for (int pageIx = 0; pageIx < m_Bean.getPageCount(); ++pageIx)
        {
            ((IPDFPageView)m_Bean.getPageView(pageIx+1)).clearTextSelection();
        }
    }

    public void valueChanged(ListSelectionEvent e)
    {
        if(e.getValueIsAdjusting())
        {
            return;
        }
        
        SwingUtilities.invokeLater(new Runnable ()
        {
            public void run() 
            {
                m_Dialog.getJlOccurences().repaint();
            }
        });
        
        // Get the list of selected rows
        Object [] selectList = m_Dialog.getJlOccurences().getSelectedValues();
        if (selectList != null)
        {
            setTextSelection (selectList);
        }
    }

    public static boolean isEmptyString(Object obj)
    {
        if (obj == null)
        {
            return true;
        }
        
        return (obj.toString ().trim ().length () == 0);
    }

}
