package jPDFViewerSamples.watermark;

import jPDFViewerSamples.SampleUtil;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import com.qoppa.pdf.IWatermark;
import com.qoppa.pdfViewer.PDFViewerBean;

public class SimpleWatermark extends JFrame
{
	private JPanel jPanel = null;
	private PDFViewerBean viewerBean = null;

	public static void main(String[] args)
	{

		SwingUtilities.invokeLater(new Runnable() {
			public void run()
			{
				SimpleWatermark sf = new SimpleWatermark();
				sf.setVisible(true);
			}
		});
	}

	/**
	 * This method initializes
	 * 
	 */
	public SimpleWatermark()
	{
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize()
	{
		this.setBounds(new Rectangle(0, 0, (int)(800 * SampleUtil.getDPIScalingMultiplier()), (int)(600 * SampleUtil.getDPIScalingMultiplier())));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setContentPane(getJPanel());
		this.setTitle("Qoppa Software - jPDFViewer Watermark Sample");
		this.setLocationRelativeTo(null);

		// Create the watermark
		IWatermark watermark = new IWatermark() {
			public void drawWatermark(Graphics2D g, int type, int pageIndex, int pageWidth, int pageHeight)
			{
				switch (type) {
				case IWatermark.TYPE_IMAGES:
				case IWatermark.TYPE_VIEWER: {
					g.setColor(new Color(128, 128, 128, 128));
					break;
				}
				case IWatermark.TYPE_PRINT: {
					// Watermark color (this can have transparency, but Java
					// then makes the spool file very large)
					g.setColor(new Color(128, 128, 128));
					break;
				}
				}

				// Set the font
				g.setFont(new Font("sansserif", Font.PLAIN, 100));

				// Draw the watermark string, rotated 45 degrees
				g.rotate(Math.toRadians(45));
				g.drawString("Watermark", 150, g.getFontMetrics().getMaxDescent());
			}
		};

		// Set the watermark on the PDFViewerBean
		getPDFViewerBean().setWatermark(watermark);
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel()
	{
		if (jPanel == null)
		{
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getPDFViewerBean(), BorderLayout.CENTER);
		}
		return jPanel;
	}

	/**
	 * This method initializes PDFViewerBean
	 * 
	 * @return com.qoppa.pdfViewer.PDFViewerBean
	 */
	private PDFViewerBean getPDFViewerBean()
	{
		if (viewerBean == null)
		{
			viewerBean = new PDFViewerBean();
		}
		return viewerBean;
	}
}
